__productname__ = 'devi'
__description__ = 'devi is a tool for managing your project templates'
__version__ = '0.1.0'
__author__ = 'the devi team'
__license__ = 'MIT'

import devi.config as config

config.init()
